#!/bin/bash -e

# made by Costin Constantin, Yocto QA Romania


################ Steps to perform in order to achieve image mods with monimal interaction #####################
#
# 1) mount hddimg in an local mount dir
# 2) cp initrd from local mount dir to a local temp dir
# 3) calculate microcode and ramfs size as initrd= cat microcode.cpio ramfs.cpio.gz and obtain both artefacts; ramfs should be available from autobuilder 
# 4) modify install.sh 
# 5) repack initrd
# 6) cp modified initrd to hddimg
# 
# In theory that's it!
#
# Notes on how to do it:
# Before everything else, you need to add your user to the visudo file, allowing sudo with no password. On Ubuntu:
# sudo visudo
# ...
# #at the end of the file add
# your_user_name  ALL=NOPASSWD:       ALL
# ...
# also, be sure to have 7 zip installed
# sudo apt-get install p7zip-full

################  VAR section #################

HDDIMG="$1"
ROOTFS="$2"
work_dir="$(pwd)"
mount_dir="${work_dir}/mnt_dir"
EFI_BOOT_dir="${mount_dir}/EFI/BOOT"
td="${work_dir}/tmp_dir" # temporary work dir where mods are applied
tmp1="${td}/tmp1"

ramfs_gz_size="$(ls -l ${work_dir}/*.cpio.gz | nawk '{print $5}')"
#inst_md5="$(md5sum install.sh | nawk '{print $1}')" # the md5sum of the install.sh script ; not implemented in final ver

####### error codes section: 
err_no_args="2" # when no args are presented in command line


###############################################

display_help(){
echo -e "Usage: img_prepare.sh file.hddimg initramfs_file.cpio.gz\n

where file.hddimg is the image we would like to tweak\n
      initramfs_file.cpio.gz is the initramfs file downloaded from autobuilder \n
      alongside the core*.hddimg file. This always has to be the image-minimal.cpio.gz\n
      as this is the only published on AB\n
"

}

create_env(){
# creating necessary dirs if not available:

  if [[ -z ${HDDIMG}  && -z ${ROOTFS} ]]
  then
    display_help
  fi


  if [[ ! -d ${mount_dir} && ! -d ${td} && ! -d ${tmp1} ]]
  then
    mkdir -p ${tmp1} ${mount_dir}
    chmod -R a+rw ${td} ${mount_dir}
  fi

  if ! ls ${tmp1} ${mount_dir} ${td}
    then
    echo "Necessary dirs cannot be created. Exiting ..."
    exit 3
  fi 


# 1) mount hddimg:
  sudo mount ${HDDIMG} ${mount_dir}
# 2) cp initrd from mount_dir to a td
  cp -p ${mount_dir}/initrd ${td}
  cd ${td}
# 3) obtain artefacts: microcode.cpio and ramfs (though we already have it)
  initrd_size="$(ls -l ${td}/initrd | nawk '{print $5}')"
  microcode_size="$((${initrd_size}-${ramfs_gz_size}))"
  dd if=initrd of=${td}/microcode.cpio bs=1 count=${microcode_size}
  dd if=initrd of=${td}/ramfs.cpio.gz bs=1 skip=${microcode_size}
  rm initrd # no longer needed 
#unpack ramfs in an temp dir
  gunzip ramfs.cpio.gz 
  mv ramfs.cpio ${tmp1}
  cd ${tmp1}
  cpio -id < ramfs.cpio
  rm ramfs.cpio
return 0
}



modify_files(){
# using a small "trick". Because install.sh is not modified an a build basis, just copy an already modified one directly into rootfs
# if not ... than modify the actual one from current rootfs
# 4) modify install script && sysconfig so that it should consider installing opt first
cd ${tmp1}
cat install.sh | while read line
		 do 
 		   if echo ${line} | egrep "^*read* answer" >/dev/null   
		   then
		     echo $line | nawk 'gsub("read answer","answer=\"y\"")' >> tmp_file
		   elif echo ${line} | egrep "^*read* enter" > /dev/null
		   then
	 	     echo $line | nawk 'gsub("read enter","#read enter")' >> tmp_file
		     echo "ifconfig eth0 192.168.99.200 netmask 255.255.255.0" >> tmp_file
		     echo "sleep 30" >> tmp_file
		   elif echo $line | egrep "menuentry" >/dev/null
		   then
		     echo 'set timeout_style=hidden' >> tmp_file
		     echo 'set timeout=5' >> tmp_file
      		     echo $line >> tmp_file
		   else 	
		     echo $line >> tmp_file
		   fi
		 done
cat tmp_file > install.sh
rm tmp_file

#for syslinux.cfg

cd ${mount_dir}
cat ${mount_dir}/syslinux.cfg | while read line
		   do 
 		     if echo $line | egrep "DEFAULT" >/dev/null   
		     then
		       echo $line | nawk 'gsub("boot","install")' >> ${td}/syslinux.cfg
		     else 	
		       echo $line >> ${td}/syslinux.cfg
		     fi
		   done
sudo cp ${td}/syslinux.cfg ${mount_dir}/syslinux.cfg
rm ${td}/syslinux.cfg

#4') for EFI:
cd ${tmp1}
cat install-efi.sh | while read line
		 do 
 		   if echo ${line} | egrep "^*read* answer" >/dev/null   
		   then
		     echo $line | nawk 'gsub("read answer","answer=\"y\"")' >> tmp_file
		   elif echo ${line} | egrep "^*read* enter" > /dev/null
		   then
	 	     echo $line | nawk 'gsub("read enter","#read enter")' >> tmp_file
		     echo "ifconfig eth0 192.168.99.200 netmask 255.255.255.0" >> tmp_file
		     echo "sleep 30" >> tmp_file
		   else 	
		     echo $line >> tmp_file
		   fi
		 done
cat tmp_file > install-efi.sh
rm tmp_file

#for grub.cfg

cd ${mount_dir}
cat ${EFI_BOOT_dir}/grub.cfg | while read line
		   do 
 		     if echo $line | egrep "default" >/dev/null   
		     then
		       echo $line | nawk 'gsub("boot","install")' >> ${td}/grub.cfg
		     else 	
		       echo $line >> ${td}/grub.cfg
		     fi
		   done
sudo cp ${td}/grub.cfg ${EFI_BOOT_dir}/grub.cfg
rm ${td}/grub.cfg


return 0
}


repack(){
# 5) repack initrd
  cd ${tmp1}
  find . | cpio -o -H newc > ../ramfs.cpio
  cd ${td} 
  7z a -mx=9 -tgzip ramfs.cpio.gz ramfs.cpio
  cat microcode.cpio ramfs.cpio.gz > initrd

# echo Ies aici
# exit 0
  sudo rm ${mount_dir}/initrd 
  sudo cp initrd ${mount_dir}
return 0
}

clean_env(){
# clean and prepare env for a new run
  sudo umount ${mount_dir}
  rm -r ${td} ${mount_dir}
return 0
}

######################## MAIN ########################

if [[ $# -ne "2" ]]
then 
  display_help
  exit $err_no_args
fi

create_env
modify_files
repack
clean_env

exit 0
